@@ -1,4 +1,4 @@
-// Copyright 2018-2021 Cargill Incorporated
+// Copyright 2018-2022 Cargill Incorporated
 //
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
@@ -848,7 +848,7 @@ mod tests {
     use sawtooth::receipt::store::diesel::DieselReceiptStore;
     use transact::{
         database::{btree::BTreeDatabase, Database},
-        families::command::workload::CommandTransactionBuilder,
+        families::command::CommandTransactionBuilder,
         protocol::command::{BytesEntry, Command, SetState},
         state::merkle::INDEXES,
     };
